#include <MP1246.h>
#include <MP1090.h>
#include <Wire.h>
#include <PinChangeInt.h>

const int Radio_PWR_Pin = 12;

const int MP1093_VOL_A = 14;
const int MP1093_VOL_B = 15;
const int MP1093_BAND_1_A = 16;
const int MP1093_BAND_1_B = 17;
const int MP1093_BAND_2_A = 2;
const int MP1093_BAND_2_B = 3;
const int MP1093_BAND_3_A = 4;
const int MP1093_BAND_3_B = 5;
const int MP1093_BAND_4_A = 6;
const int MP1093_BAND_4_B = 7;
const int MP1093_BAND_5_A = 8;
const int MP1093_BAND_5_B = 9;

const int MP1093_CounterClockWiseRot = 0;
const int MP1093_ClockWiseRot = 1;
const int MP1093_Unchanged = 2;

volatile byte VOLUME_FLAG = MP1093_Unchanged;
volatile byte BAND_1_FLAG = MP1093_Unchanged;
volatile byte BAND_2_FLAG = MP1093_Unchanged;
volatile byte BAND_3_FLAG = MP1093_Unchanged;
volatile byte BAND_4_FLAG = MP1093_Unchanged;
volatile byte BAND_5_FLAG = MP1093_Unchanged;

void setup()
{
  Serial.begin (9600);
  Wire.begin();
  
  PCintPort::attachInterrupt(MP1093_VOL_A, &MP1093_VOL_A_ISR, RISING);
  PCintPort::attachInterrupt(MP1093_VOL_B, &MP1093_VOL_B_ISR, RISING);
  PCintPort::attachInterrupt(MP1093_BAND_1_A, &MP1093_BAND_1_A_ISR, RISING);
  PCintPort::attachInterrupt(MP1093_BAND_1_B, &MP1093_BAND_1_B_ISR, RISING);
  PCintPort::attachInterrupt(MP1093_BAND_2_A, &MP1093_BAND_2_A_ISR, RISING);
  PCintPort::attachInterrupt(MP1093_BAND_2_B, &MP1093_BAND_2_B_ISR, RISING);
  PCintPort::attachInterrupt(MP1093_BAND_3_A, &MP1093_BAND_3_A_ISR, RISING);
  PCintPort::attachInterrupt(MP1093_BAND_3_B, &MP1093_BAND_3_B_ISR, RISING);
  PCintPort::attachInterrupt(MP1093_BAND_4_A, &MP1093_BAND_4_A_ISR, RISING);
  PCintPort::attachInterrupt(MP1093_BAND_4_B, &MP1093_BAND_4_B_ISR, RISING);
  PCintPort::attachInterrupt(MP1093_BAND_5_A, &MP1093_BAND_5_A_ISR, RISING);
  PCintPort::attachInterrupt(MP1093_BAND_5_B, &MP1093_BAND_5_B_ISR, RISING);
  
    
  MP1090::Init(Radio_PWR_Pin);
  MP1090::SetStation(95200);
  MP1090::SetVolume(3);
  
  MP1246::Init ();
  MP1246::SetVolume (0x2F);
}
//--------------------------------------------------------
void loop ()
{
  if ((VOLUME_FLAG == MP1093_ClockWiseRot) || (VOLUME_FLAG == MP1093_CounterClockWiseRot))
  {
    ChangeVolume ();
    Serial.print ("Volume   ");
    Serial.print (MP1246::GetVolume (), DEC);
    Serial.print ("\n");
    VOLUME_FLAG = MP1093_Unchanged;
  }
  
  if ((BAND_1_FLAG == MP1093_ClockWiseRot) || (BAND_1_FLAG == MP1093_CounterClockWiseRot))
  {
    ChangeBand (1, BAND_1_FLAG);
    Serial.print ("BAND_1   ");
    Serial.print (MP1246::GetBandValue (1), DEC);
    Serial.print ("\n");
    BAND_1_FLAG = MP1093_Unchanged;
  }
  
  if ((BAND_2_FLAG == MP1093_ClockWiseRot) || (BAND_2_FLAG == MP1093_CounterClockWiseRot))
  {
    ChangeBand (2, BAND_2_FLAG);
    Serial.print ("BAND_2   ");
    Serial.print (MP1246::GetBandValue (2), DEC);
    Serial.print ("\n");
    BAND_2_FLAG = MP1093_Unchanged;
  }
  
  if ((BAND_3_FLAG == MP1093_ClockWiseRot) || (BAND_3_FLAG == MP1093_CounterClockWiseRot))
  {
    ChangeBand (3, BAND_3_FLAG);
    Serial.print ("BAND_3   ");
    Serial.print (MP1246::GetBandValue (3), DEC);
    Serial.print ("\n");
    BAND_3_FLAG = MP1093_Unchanged;
  }
  
  if ((BAND_4_FLAG == MP1093_ClockWiseRot) || (BAND_4_FLAG == MP1093_CounterClockWiseRot))
  {
    ChangeBand (4, BAND_4_FLAG);
    Serial.print ("BAND_4   ");
    Serial.print (MP1246::GetBandValue (4), DEC);
    Serial.print ("\n");
    BAND_4_FLAG = MP1093_Unchanged;
  }
  
  if ((BAND_5_FLAG == MP1093_ClockWiseRot) || (BAND_5_FLAG == MP1093_CounterClockWiseRot))
  {
    ChangeBand (5, BAND_5_FLAG);
    Serial.print ("BAND_5   ");
    Serial.print (MP1246::GetBandValue (5), DEC);
    Serial.print ("\n");
    BAND_5_FLAG = MP1093_Unchanged;
  }
}
//--------------------------------------------------------
void MP1093_VOL_A_ISR ()
{
  if (PCintPort::arduinoPin == MP1093_VOL_A)
  {
   delay (100);
   
   if ((digitalRead (MP1093_VOL_A)) && (!digitalRead (MP1093_VOL_B)))
   {
     VOLUME_FLAG = MP1093_ClockWiseRot;
   }
  }
}
//-----------------------------------------------------------------
void MP1093_VOL_B_ISR ()
{
  if (PCintPort::arduinoPin == MP1093_VOL_B)
  {
    delay (100);
    if ((digitalRead (MP1093_VOL_B)) && (!digitalRead (MP1093_VOL_A)))
    {
      VOLUME_FLAG = MP1093_CounterClockWiseRot;
    }
  }
}
//-----------------------------------------------------------------
void MP1093_BAND_1_A_ISR ()
{
  if (PCintPort::arduinoPin == MP1093_BAND_1_A)
  {
     delay (100);
     if ((digitalRead (MP1093_BAND_1_A)) && (!digitalRead (MP1093_BAND_1_B)))
     {
        BAND_1_FLAG = MP1093_ClockWiseRot;
     }
  }
}
//-----------------------------------------------------------------
void MP1093_BAND_1_B_ISR ()
{
  if (PCintPort::arduinoPin == MP1093_BAND_1_B)
  {
   delay (100);
   if ((digitalRead (MP1093_BAND_1_B)) && (!digitalRead (MP1093_BAND_1_A)))
   {
     BAND_1_FLAG = MP1093_CounterClockWiseRot;
   }
  }
}
//-----------------------------------------------------------------
void MP1093_BAND_2_A_ISR ()
{
  if (PCintPort::arduinoPin == MP1093_BAND_2_A)
  {
     delay (100);
     if ((digitalRead (MP1093_BAND_2_A)) && (!digitalRead (MP1093_BAND_2_B)))
     {
        BAND_2_FLAG = MP1093_ClockWiseRot;
     }
  }
}
//-----------------------------------------------------------------
void MP1093_BAND_2_B_ISR ()
{
  if (PCintPort::arduinoPin == MP1093_BAND_2_B)
  {
   delay (100);
   if ((digitalRead (MP1093_BAND_2_B)) && (!digitalRead (MP1093_BAND_2_A)))
   {
     BAND_2_FLAG = MP1093_CounterClockWiseRot;
   }
  }
}
//-----------------------------------------------------------------
void MP1093_BAND_3_A_ISR ()
{
  if (PCintPort::arduinoPin == MP1093_BAND_3_A)
  {
     delay (100);
     if ((digitalRead (MP1093_BAND_3_A)) && (!digitalRead (MP1093_BAND_3_B)))
     {
        BAND_3_FLAG = MP1093_ClockWiseRot;
     }
  }
}
//-----------------------------------------------------------------
void MP1093_BAND_3_B_ISR ()
{
  if (PCintPort::arduinoPin == MP1093_BAND_3_B)
  {
   delay (100);
   if ((digitalRead (MP1093_BAND_3_B)) && (!digitalRead (MP1093_BAND_3_A)))
   {
     BAND_3_FLAG = MP1093_CounterClockWiseRot;
   }
  }
}
//-----------------------------------------------------------------
void MP1093_BAND_4_A_ISR ()
{
  if (PCintPort::arduinoPin == MP1093_BAND_4_A)
  {
     delay (100);
     if ((digitalRead (MP1093_BAND_4_A)) && (!digitalRead (MP1093_BAND_4_B)))
     {
        BAND_4_FLAG = MP1093_ClockWiseRot;
     }
  }
}
//-----------------------------------------------------------------
void MP1093_BAND_4_B_ISR ()
{
  if (PCintPort::arduinoPin == MP1093_BAND_4_B)
  {
   delay (100);
   if ((digitalRead (MP1093_BAND_4_B)) && (!digitalRead (MP1093_BAND_4_A)))
   {
     BAND_4_FLAG = MP1093_CounterClockWiseRot;
   }
  }
}
//-----------------------------------------------------------------
void MP1093_BAND_5_A_ISR ()
{
  if (PCintPort::arduinoPin == MP1093_BAND_5_A)
  {
     delay (100);
     if ((digitalRead (MP1093_BAND_5_A)) && (!digitalRead (MP1093_BAND_5_B)))
     {
        BAND_5_FLAG = MP1093_ClockWiseRot;
     }
  }
}
//-----------------------------------------------------------------
void MP1093_BAND_5_B_ISR ()
{
  if (PCintPort::arduinoPin == MP1093_BAND_5_B)
  {
   delay (100);
   if ((digitalRead (MP1093_BAND_5_B)) && (!digitalRead (MP1093_BAND_5_A)))
   {
     BAND_5_FLAG = MP1093_CounterClockWiseRot;
   }
  }
}
//-----------------------------------------------------------------
void ChangeVolume ()
{ 
  byte CurValue = MP1246::GetVolume ();
  byte NewValue = CurValue;
  
  if (VOLUME_FLAG == MP1093_ClockWiseRot)
  {
    if (NewValue > 0)
     NewValue--;
  }
  
  else if (VOLUME_FLAG == MP1093_CounterClockWiseRot)
  {
    if (NewValue < 0x2F)
      NewValue++;
  }
  
  if (NewValue != CurValue)
    MP1246::SetVolume (NewValue);
}
//-----------------------------------------------------------------
void ChangeBand (byte Band, byte BAND_FLAG)
{
  byte CurValue = MP1246::GetBandValue (Band);
  byte NewValue = CurValue;

  if (BAND_FLAG == MP1093_ClockWiseRot)
  {
    if (NewValue > 0)
     NewValue--;
  }
  
  else if (BAND_FLAG == MP1093_CounterClockWiseRot)
  {
    if (NewValue < 15)
      NewValue++;
  }
  
  if (NewValue != CurValue)
    MP1246::SetBand (Band, NewValue);
}
//-----------------------------------------------------------------
